@@ -10,7 +10,7 @@ except ImportError:
 from PyQt5.QtCore import pyqtSignal,Qt
 from PyQt5.QtWidgets import (QApplication, QWidget,QToolTip,QPushButton,QMessageBox,QDesktopWidget,QMainWindow,
                              QVBoxLayout,QHBoxLayout,QGridLayout,QTextEdit,QLabel,QRadioButton,QCheckBox,
-                             QLineEdit,QGroupBox,QSplitter)
+                             QLineEdit,QGroupBox,QSplitter,QFileDialog)
 from PyQt5.QtGui import QIcon,QFont,QTextCursor,QPixmap
 import serial
 import serial.tools.list_ports
@@ -245,12 +245,21 @@ class MainWindow(QMainWindow):
         settingLayout.setStretch(2, 2.5)
 
         # right functional layout
+        self.filePathWidget = QLineEdit()
+        self.openFileButton = QPushButton("Open File")
+        self.sendFileButton = QPushButton("Send File")
+        self.clearHistoryButton = QPushButton("Clear History")
         self.addButton = QPushButton(parameters.strAdd)
-        functionalGroupBox = QGroupBox(parameters.strFunctionalSend)
-        functionalGridLayout = QGridLayout()
-        functionalGridLayout.addWidget(self.addButton,0,1)
-        functionalGroupBox.setLayout(functionalGridLayout)
-        sendFunctionalLayout.addWidget(functionalGroupBox)
+        fileSendGroupBox = QGroupBox(parameters.strSendFile)
+        fileSendGridLayout = QGridLayout()
+        fileSendGridLayout.addWidget(self.filePathWidget, 0, 0, 1, 1)
+        fileSendGridLayout.addWidget(self.openFileButton, 0, 1, 1, 1)
+        fileSendGridLayout.addWidget(self.sendFileButton, 1, 0, 1, 2)
+        fileSendGroupBox.setLayout(fileSendGridLayout)
+        sendFunctionalLayout.addWidget(fileSendGroupBox)
+        sendFunctionalLayout.addWidget(self.clearHistoryButton)
+        sendFunctionalLayout.addWidget(self.addButton)
+        sendFunctionalLayout.addStretch(1)
         self.isHideFunctinal = True
         self.hideFunctional()
 
@@ -275,7 +284,7 @@ class MainWindow(QMainWindow):
         if sys.platform == "win32":
             ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID("comtool")
         self.show()
-        print("config file path:",os.getcwd()+"/comtool.settings.config")
+        print("config file path:",parameters.configFilePath)
 
     def initEvent(self):
         self.serialOpenCloseButton.clicked.connect(self.openCloseSerial)
@@ -293,6 +302,9 @@ class MainWindow(QMainWindow):
         self.settingsButton.clicked.connect(self.showHideSettings)
         self.skinButton.clicked.connect(self.skinChange)
         self.aboutButton.clicked.connect(self.showAbout)
+        self.openFileButton.clicked.connect(self.selectFile)
+        self.sendFileButton.clicked.connect(self.sendFile)
+        self.clearHistoryButton.clicked.connect(self.clearHistory)
         self.addButton.clicked.connect(self.functionAdd)
         self.functionalButton.clicked.connect(self.showHideFunctional)
         self.sendArea.currentCharFormatChanged.connect(self.sendAreaFontChanged)
@@ -334,10 +346,9 @@ class MainWindow(QMainWindow):
                         self.com.dtr = False
                     else:
                         self.com.dtr = True
-                    self.serialOpenCloseButton.setDisabled(True)
                     self.com.open()
-                    print("open success")
-                    print(self.com)
+                    # print("open success")
+                    # print(self.com)
                     self.setDisableSettingsSignal.emit(True)
                     self.receiveProcess = threading.Thread(target=self.receiveData)
                     self.receiveProcess.setDaemon(True)
@@ -346,7 +357,7 @@ class MainWindow(QMainWindow):
                     self.com.close()
                     self.receiveProgressStop = True
                     self.errorSignal.emit( parameters.strOpenFailed +"\n"+ str(e))
-                    self.serialOpenCloseButton.setDisabled(False)
+                    self.setDisableSettingsSignal.emit(False)
         except Exception as e:
             print(e)
     
@@ -413,8 +424,8 @@ class MainWindow(QMainWindow):
                 data = self.getSendData()
                 if data == -1:
                     return
-                print(self.sendArea.toPlainText())
-                print("send:",data)
+                # print(self.sendArea.toPlainText())
+                # print("send:",data)
                 self.sendCount += len(data)
                 self.com.write(data)
                 data = self.sendArea.toPlainText()
@@ -430,7 +441,7 @@ class MainWindow(QMainWindow):
                         t.start()
         except Exception as e:
             self.errorSignal.emit(parameters.strWriteError)
-            print(e)
+            # print(e)
 
     def scheduledSend(self):
         self.isScheduledSending = True
@@ -468,12 +479,14 @@ class MainWindow(QMainWindow):
                     else:
                         self.receiveUpdateSignal.emit(bytes.decode(self.encodingCombobox.currentText(),"ignore"))
             except Exception as e:
-                print("receiveData error")
+                # print("receiveData error")
                 # if self.com.is_open and not self.serialPortCombobox.isEnabled():
                 #     self.openCloseSerial()
                 #     self.serialPortCombobox.clear()
                 #     self.detectSerialPort()
-                print(e)
+                if 'multiple access' in str(e):
+                    self.errorSignal.emit("device disconnected or multiple access on port?")
+                break
             # time.sleep(0.009)
 
     def updateReceivedDataDisplay(self,str):
@@ -505,7 +518,7 @@ class MainWindow(QMainWindow):
                 self.sendArea.insertPlainText(data)
         except Exception as e:
             # QMessageBox.information(self,parameters.strWriteFormatError,parameters.strWriteFormatError)
-            print("format error");
+            print("format error")
 
     def sendHistoryIndexChanged(self):
         self.sendArea.clear()
@@ -565,6 +578,8 @@ class MainWindow(QMainWindow):
                 self.serialPortCombobox.clear()
                 for i in portList:
                     showStr = str(i[0])+" "+str(i[1])
+                    if i[0].startswith("/dev/cu.Bluetooth-Incoming-Port"):
+                        continue
                     self.serialPortCombobox.addItem(showStr)    
                 index = self.serialPortCombobox.findText(currText)
                 if index>=0:
@@ -597,7 +612,7 @@ class MainWindow(QMainWindow):
             data = bytes.fromhex(data)
         except Exception:
             return -1
-        print(data)
+        # print(data)
         return data
 
     def programExitSaveParameters(self):
@@ -633,7 +648,7 @@ class MainWindow(QMainWindow):
         else:
             paramObj.dtr = 0
         paramObj.encodingIndex = self.encodingCombobox.currentIndex()
-        f = open("comtool.settings.config","wb")
+        f = open(parameters.configFilePath,"wb")
         f.truncate()
         pickle.dump(paramObj, f)
         pickle.dump(paramObj.sendHistoryList,f)
@@ -642,12 +657,12 @@ class MainWindow(QMainWindow):
     def programStartGetSavedParameters(self):
         paramObj = parameters.ParametersToSave()
         try:
-            f = open("comtool.settings.config", "rb")
+            f = open(parameters.configFilePath, "rb")
             paramObj = pickle.load( f)
             paramObj.sendHistoryList = pickle.load(f)
             f.close()
         except Exception as e:
-            f = open("comtool.settings.config", "wb")
+            f = open(parameters.configFilePath, "wb")
             f.close()
         self.serailBaudrateCombobox.setCurrentIndex(paramObj.baudRate)
         self.serailBytesCombobox.setCurrentIndex(paramObj.dataBytes)
@@ -758,6 +773,36 @@ class MainWindow(QMainWindow):
                                 '</h1><br><b style="color:#08c7a1;margin = 5px;">V'+str(helpAbout.versionMajor)+"."+
                                 str(helpAbout.versionMinor)+"."+str(helpAbout.versionDev)+
                                 "</b><br><br>"+helpAbout.date+"<br><br>"+helpAbout.strAbout())
+    def selectFile(self):
+        oldPath = self.filePathWidget.text()
+        if oldPath=="":
+            oldPath = os.getcwd()
+        fileName_choose, filetype = QFileDialog.getOpenFileName(self,  
+                                    "SelectFile",
+                                    oldPath,
+                                    "All Files (*);;")
+
+        if fileName_choose == "":
+            return
+        self.filePathWidget.setText(fileName_choose)
+
+    def sendFile(self):
+        filename = self.filePathWidget.text()
+        if not os.path.exists(filename):
+            self.errorSignal.emit("File path error\npath:%s" %(filename))
+            return
+        try:
+            f = open(filename, "rb")
+        except Exception as e:
+            self.errorSignal.emit("Open file %s failed! \n %s" %(filename, str(e)))
+            return
+        self.com.write(f.read()) #TODO: optimize send in new thread
+        f.close()
+
+    def clearHistory(self):
+        self.param.sendHistoryList.clear()
+        self.sendHistory.clear()
+        self.errorSignal.emit("History cleared!")
 
     def autoUpdateDetect(self):
         auto = autoUpdate.AutoUpdate()
