@@ -1,3 +1,5 @@
+import sys, os
+
 appName = "COMTool"
 strDataDirName = "share/COMToolData"
 strDataAssetsDirName = "share/COMToolData/assets"
@@ -29,6 +31,7 @@ strSerialSendSettings = "Send Settings"
 strClearReceive = "ClearReceive"
 strAdd = "+"
 strFunctionalSend = "Functional Send"
+strSendFile = "Send File"
 strBaudRateDefault = "115200"
 strOpenFailed = "Open Failed"
 strOpenReady = "Open Ready"
@@ -45,6 +48,20 @@ strNeedUpdate = "Need Update"
 strUpdateNow = "update now?"
 strUninstallApp = "uninstall app"
 
+configFileName="comtool.settings.config"
+configFilePath=configFileName
+
+if sys.platform.startswith('linux') or sys.platform.startswith('darwin') or sys.platform.startswith('freebsd'):
+    configFileDir = os.path.join(os.getenv("HOME"), ".config/comtool")
+    try:
+        configFilePath = os.path.join(configFileDir, configFileName)
+        if not os.path.exists(configFileDir):
+            os.makedirs(configFileDir)
+    except:
+        pass
+else:
+    configFilePath  = os.path.join(os.getcwd(), configFileName)
+
 
 class ParametersToSave:
     baudRate = 4
